package visl.rasmus.gui;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

/** Implements a bar that ranges from 0 to 100 with 0 = green and 100 = red */
public class ProgressOMeter extends Canvas {
    static public final int DOWN = 0, UP = 1, LEFT = 2, RIGHT = 3;

    protected float level;
    protected Dimension dim;
    protected int min, max, critical, type;
    protected Color badcolor, goodcolor;

    public ProgressOMeter() {
	this(0, 75, 100, UP);
    }
    public ProgressOMeter(int min, int max) {
	this(min, min+(max-min)/2, max, UP);
    }
    public ProgressOMeter(int min, int critical, int max, int type) { 
	this.min = min;
	this.critical = critical;
	this.max = max;
	this.type = type;
	level = min;
	dim = null;
    }
    public void resize(Dimension d) {
	super.resize(d);
	this.dim = d;
    }
    public Dimension preferredSize() {
	if (dim != null) return dim;
	return new Dimension(10,100);
    }
    public Dimension minimumSize() { return preferredSize(); }
    public void paint(Graphics g) {
	Dimension d = size();
	int range = 0;
	if (type == UP || type == DOWN)
	    range = d.height;
	else
	    range = d.width;

	float lvl = (level-min)*100/(max-min);
	float crit = (critical-min)*100/(max-min);
	int h = (int)(lvl*(range-2)/100);
	int cr = (int)(crit*(range-2)/100);
	
	float goodstep = 200f/cr;
	float badstep = 55f/(range-cr);

	g.setColor(Color.white);
	g.drawRect(0,0, d.width-1, d.height-1);
	float gr = 255, r = 255;
	for (int i = 0; i < h; i++) {
	    if (gr > 255) gr = 255;
	    if (r > 255) r = 255;
	    g.setColor(new Color((int)r, (int)gr, 0));
	    if (i < cr) {
		if (r < 255) r = r + goodstep;
		else gr = gr - goodstep; 
	    } else {
		gr = gr - badstep;
	    }
	    switch (type) {
	    case DOWN:
		g.drawLine(1, i+2, d.width-2, i+2);
		break;
	    case UP:
		g.drawLine(1, d.height-(i+2), d.width-2, d.height-(i+2));
		break;
	    case LEFT:
		g.drawLine(i+2, 1, i+2, d.height-2);
		break;
	    case RIGHT:
		g.drawLine(d.width-(i+2), 1, d.width-(i+2), d.height-2);
		break;
	    }
	}
	g.setColor(Color.white);
	g.drawLine(1, d.height-(cr+2), d.width-2, d.height-(cr+2));
    }
    /** Sets the current level of the progress-o-meter, 0 - 100 */
    public void setLevel(float level) {
	this.level = (level > max ? max : (level < min ? min : level));
	repaint();
    }
    public float getLevel() { 
	return level; 
    }
    public void setRange(int min, int max) {
	this.min = min;
	this.max = max;
    }
    public void setCritical(int critical) {
	if (critical >= max) critical = max-1;
	if (critical <= min) critical = min+1;
	this.critical = critical;
    }
    public int getCritical() { return critical; }
}
