package visl.rasmus.handler;

import java.awt.Color;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;

/** This class implements a unversal CursorHandler, that enables an AWT
 * component to use images as cursors.
 * All components must call the cursorhandlers updateCursor method from their
 * handleEvent method.
 * @author Rasmus Lock Larsen, VISL Februar 2001. */
public class CursorHandler
	implements java.awt.image.ImageObserver, GraphicsHandler {
	protected Image cursor;
	protected int curs_x, curs_y, offs_x, offs_y; // Temp vars.
	protected Color background;
	protected Frame parent;
	protected Event e;

	public CursorHandler(Image cursor, Frame parent) {
		this.cursor = cursor;
		this.parent = parent;
		background = Color.black;
		if (cursor != null) {
			offs_x = cursor.getWidth(this);
			offs_y = cursor.getHeight(this);
		} else {
			offs_x = 0;
			offs_y = 0;
		}
		curs_x = 0;
		curs_y = 0;
	}
	public void updateGraphics(Graphics g, Object arg) {
		e = null;
		if (arg == null) {
			e =
				new Event(
					null,
					0,
					Event.MOUSE_ENTER,
					curs_x,
					curs_y,
					0,
					0,
					null);
		} else {
			e = (Event) arg;
		}
		if (e.id == Event.MOUSE_MOVE || e.id == Event.MOUSE_DRAG) {
			if (cursor != null) {
				g.setXORMode(background);
				// Remove old cursor
				g.drawImage(cursor, curs_x, curs_y, this);
				curs_x = e.x - offs_x / 2;
				curs_y = e.y - offs_y / 2;
				// Draw new cursor
				g.drawImage(cursor, curs_x, curs_y, this);
			}
		} else if (e.id == Event.MOUSE_ENTER && parent != null) {
			parent.setCursor(Frame.CROSSHAIR_CURSOR);
			// Draw cursor
			g.setXORMode(background);
			g.drawImage(cursor, curs_x, curs_y, this);
			g.setPaintMode();
		} else if (e.id == Event.MOUSE_EXIT && parent != null) {
			parent.setCursor(Frame.DEFAULT_CURSOR);
			// Remove old cursor
			g.setXORMode(background);
			g.drawImage(cursor, curs_x, curs_y, this);
		}
	}
	public boolean imageUpdate(
		Image img,
		int infoflags,
		int x,
		int y,
		int width,
		int height) {
		return true;
	}
}
