package visl.rasmus.handler;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.util.Random;

import visl.rasmus.utils.RandomGenerator;

/** Implements transparent labels to be used with MultiHandler */
public class MultiLabel extends MultiComponent {
    public static final int 
	UNDERLINE = 0x1,
	COLORCHANGE = 0x2;
    protected String text;
    protected int w, h, attr;
    public MultiLabel(String s) {
	text = s;
	attr = COLORCHANGE;
	Font f = getFont();
	FontMetrics fm = null;
	if (f != null) fm = getFontMetrics(f);
	if (fm != null) {
	    w = fm.stringWidth(text);
	    h = fm.getHeight();
	} else {
	    w = 0;
	    h = 0;
	}
	setFont(new Font("Helvetica", Font.BOLD, 16));
    }
    public String getText() { return text; }
    public void setText(String s) { text = s; }
    public void setAttribute(int attrib) {	
	attr = (attr & ~attrib) + attrib;
    }
    public void clearAttribute(int attrib) {
	attr = (attr & ~attrib);
    }
    public void paint(Graphics g) {
	if (w == 0 || h == 0) size();
	Color col = g.getColor();
	if (isAttribute(COLORCHANGE)) {
	    Color c = getForeground();
	    if (c == null) {
		Random randgen = RandomGenerator.instance();
		c = new Color(randgen.nextFloat(), 
			      randgen.nextFloat(),
			      randgen.nextFloat());
	    }
	    float hsbs[] = new float[3];
	    Color.RGBtoHSB(c.getRed(),c.getGreen(), c.getBlue(), hsbs);
	    hsbs[0] += 0.02f;
	    if (hsbs[0] > 1) {
		hsbs[0] = 0;
	    }
	    setForeground(Color.getHSBColor(hsbs[0], hsbs[1], hsbs[2]));
	}
	g.setColor(getForeground());
	if (getFont() != null) 
	    g.setFont(getFont());
	if (isAttribute(UNDERLINE))
	    g.drawLine(x - w/2, y+2, x + w/2, y+2);
	g.drawString(text, x - w/2, y - h/2);
	g.setColor(col);
    }
    public Dimension size() {
	FontMetrics fm = getFontMetrics(getFont());
	if (fm != null) {
	    w = fm.stringWidth(text);
	    h = fm.getHeight();
	}
	return new Dimension(w,h);
    }
    public void setFont(Font f) {
	super.setFont(f);
	// Update dimensions...
	FontMetrics fm = getFontMetrics(f);
	if (fm != null) {		
	    w = fm.stringWidth(text);
	    h = fm.getHeight();
	}
    }
    public boolean isAttribute(int flag) {
	return ((attr & flag) != 0);
    }
}
