package visl.rasmus.hitserver.data;

/**
* This specifies the interface for the cretion of complex objects that have 
* a tree structure.
*
* This is the Builder participant in a Builder pattern.
* @author Martin Carlsen, VISL
* @author Rasmus Lock Larsen, VISL 5th of april 2001
*/
public interface TreeBuilder {
    /** Initialize the creation process. */
    public void beginTree();
    /** Finalize the creation process. */
    public void endTree();
    /**
     * Begin creating a subtree under the current parent node.
     * A specified node becomes the root of the subtree. 
     * That node also becomes the current parent node in the creation process.
     * @param s The specification of the contents of the root node of 
     * the subtree.
     */
    public void beginSubtree(String s);
    /**
     * Finalize the creation of a subtree.
     * The parent node of the current parent node becomes the new parent 
     * node in the creation process.
     */
    public void endSubtree();
}
