package visl.rasmus.hitserver.data;

/**
* An object of this class directs the creation of an object that has a tree 
* structure. 
* The <TT>TreeBuilder</TT> interface is used for this. A TreeBuilderDirector 
* object can be used for more than one creation.
*
* This is the Director participant in a Builder pattern.
* @author Martin Carlsen, VISL
* @author Rasmus Lock Larsen, VISL 5th of april 2001
*/
public class TreeBuilderDirector {
    TreeBuilder builder;
    
    private TreeBuilderDirector(){}
    
    /**
     * Create a TreeBuilderDirector configured with a specific TreeBuilder.
     */
    public TreeBuilderDirector(TreeBuilder tb) {
	builder = tb;
    }
    
    /**
     * Build the tree.
     */
    public void construct(String[] nodes) {
	// initialize the builder.
	builder.beginTree();
	
	if (nodes.length == 0) {
	    // The tree is empty.
	    // Don't attempt to build anything.
	} else {
	    // Build the tree, according to nodes[].
	    
	    builder.beginSubtree(nodes[0]); // the root
	    
	    for(int i=1; i<nodes.length; i++) {
		try {
		    // Find the difference in levels between new node and 
		    // previous node:
		    int dif = nodes[i-1].lastIndexOf('=') - 
			nodes[i].lastIndexOf('='); 
		    
		    // Test for level skipping,
		    // i.e. descending the tree more than one depth level 
		    // at a time, like this:
		    //
		    // =This node is on level 1
		    // ===The next node skips level 2 and tries to place 
		    // itself on level 3
		    //
		    if(dif < -1) throw new Exception("level skipping, "+dif); 
		    
		    // Go up a number of levels to the parent of the new node.
		    // (Possibly none, if the current parent is the 
		    //  new node's parent)
		    for(; dif>=0; dif--) builder.endSubtree();
		    
		    // begin a new subtree here, with the new node as root.
		    builder.beginSubtree(nodes[i]);
		}
		catch(Exception e) {
		    //System.out.println("Error!");
		    //System.out.println(nodes[i-1]);
		    //System.out.println(nodes[i]);
		    //System.out.println(nodes[i+1]);
		    throw new RuntimeException("Syntax error in analysis"+
					       "(line " + (i+1) + ": " + 
					       nodes[i]+" or before):" + e
					       );
		}
	    }
	}
	// don't forget to end rightmost subtrees, from leaf to root.
	for (int i = nodes[nodes.length-1].lastIndexOf('=')+1; i>=0; i--) 
	    builder.endSubtree();
	// finally, end the tree.
	builder.endTree();
    }
}
