package visl.rasmus.hitserver.data;

import java.io.DataInputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;

public class TreeDB {
    protected Vector items;
    protected Random randgen;
    protected String encoding = "UTF8";

    public TreeDB() {
	items = new Vector();
	randgen = new Random(System.currentTimeMillis());
    }
    /** Read the trees in filename, returns the number of errors */
    public int readTrees(String filename) {
	int numerr = 0;
	try {
	    TreeNodeBuilder builder = new TreeNodeBuilder();
	    TreeBuilderDirector director = new TreeBuilderDirector(builder);
	    URL url = new URL(filename);
	    DataInputStream din = new DataInputStream(url.openStream());
	    String s = din.readLine();
	    s = new String(s.getBytes(), encoding);
	    Vector v = new Vector();
	    boolean isreading = false, isdone = false, isroot = false;
	    int numana = 1, linenum = 1;
	    while (s != null) {
		if (s.equals("&&")) {
		    s = din.readLine();
		    s = new String(s.getBytes(), encoding);
		    linenum++;
		}
		if (s.equals("A"+numana)) {
		    if (isreading && numana == 1) isdone = true;
		    else {
			// Ambiguity detected - discard the tree.
			v.removeAllElements();
		    }
		    isreading = true;
		    isroot = true;
		    numana++;
		    s = din.readLine();
		    linenum++;
		} else if (s.equals("") && isreading) {
		    if (isreading && numana == 2) isdone = true;
		    else {
			// Ambiguity detected - discard the tree.
			v.removeAllElements();
		    }
		    isreading = false;
		    numana = 1;
		}
		if (isreading && !isdone) {
		    // Add element
		    if (s.indexOf("\t") == -1 ||
			s.indexOf("\t") == s.length()-1) {
			if (!isroot) v.addElement("="+s);
			else v.addElement(s);
			
		    } else {
			// Add parent
			String p = s.substring(0, s.indexOf("\t"));
			v.addElement((isroot?"":"=")+p);
			// Add 
			v.addElement(p.substring(0, p.lastIndexOf('=')+1)+"="+
				     (isroot?"":"=")+
				     s.substring(s.indexOf("\t")+1));
		    }
		    isroot = false;
		}
		if (isdone) {
		    String strings[] = new String[v.size()];
		    for (int i = 0; i < strings.length; i++) {
			strings[i] = (String)v.elementAt(i);
		    }
		    try {
			director.construct(strings);
			TreeNode n = builder.getTreeNode();
			n.filter();
			items.addElement(n);
		    } catch (RuntimeException x) {
			//System.out.println("Error in analysis A"+numana+
			//" before line " + linenum);
			numerr++;
		    }
		    v = new Vector();
		    isdone = false;
		    if (isreading) {
			v.addElement(s);
			isroot = false;
		    }
		}
		s = din.readLine();
		linenum++;
	    }
	    din.close();
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return numerr;
    }
    public int size() { return items.size(); }
    public Enumeration getAll() { return items.elements(); }
    public TreeNode getTree(int ix) { 
	if (ix >= 0 && ix < items.size()) return (TreeNode)items.elementAt(ix);
	return null;
    }
    public TreeNode getRandomTree() {
	return (TreeNode)items.
	    elementAt((int)(randgen.nextDouble()*(items.size()-1)));
    }
    /** Returns a random not discontinous tree with between min and max
     * words */
    public TreeNode getRandomTree(int min, int max) {
	int ix = (int)(randgen.nextDouble()*(items.size()-1));
	TreeNode n = (TreeNode)items.elementAt(ix);
	int cnt = 1;
	int size = n.leafCount();
	int msize = items.size();
	while (!(size >= min && size <= max && !n.isDiscontinousTree()) &&
	       cnt < msize){
	    n = (TreeNode)items.elementAt((ix+cnt) % msize);
	    cnt++;
	    size = n.leafCount();
	}
	if (size < min || size > max) return null;
	return n;
    }
    /** Filters all forms into VISL LITE notation - used by the games */
    public static String filterForm(String f) {
	if (f.indexOf('-') != -1) f = f.substring(0,f.indexOf('-'));
	if (f.indexOf('(') != -1) f = f.substring(0,f.indexOf('('));
	if (f.equals("in")) f = "intj";
	else if (f.equals("prop")) f = "n";
	else if (f.equals("fcl")) f = "cl";
	return f;
    }
    /** Filters all the functions into VISL LITE notation - used by the games*/
    public static String filterFunction(String F) {
	if (F.equals("Ss")) F = "Sf";
	else if (F.equals("fC")) F = "Cs";
	else if ("|STA|QUE|COM|EXC|PER|".indexOf("|"+F+"|") != -1) F = "UTT";
	return F;
    }
    public void clear() { items.removeAllElements(); }
}
