package visl.rasmus.hitserver.data;

/**
* An object of this class constructs and assembles a <TT>TreeNode</TT> by 
* implementing the <TT>TreeBuilder</TT> interface.
*
* This is a ConcreteBuilder participant in a Builder(97) pattern.
*
* @author Martin Carlsen, VISL
* @author Rasmus Lock Larsen, VISL 5th of april 2001
*/
public class TreeNodeBuilder implements TreeBuilder {
    static protected long nextid = 0;
    TreeNode current;
    TreeNode root;
    
    public void beginTree() {
	current = null;
	root = null;
    }
    
    public void endTree() {
    }
    
    public void beginSubtree(String s) {
	if (root == null) {
	    // this is the root
	    root = new TreeNode(s, nextid++);
	    current = root;
	} else {
	    TreeNode newnode = new TreeNode(s, nextid++);
	    // add the node to the current parent.
	    current.appendChild(newnode); 
	    // The new node becomes the current parent.
	    current = newnode; 
	    //newnode.doDiscontinuities(); // connect the node to discontinuities.
	}
    }
    public void endSubtree() {
	current.doDiscontinuities();
	current = (TreeNode) current.getParent();
    }
    
    /**
     * Get the result of the creational process.
     */
    public TreeNode getTreeNode() {
	return root;
    }
}







