package visl.rasmus.hitserver.gui;

import java.awt.Button;
import java.awt.Event;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import visl.rasmus.gui.Separator;
import visl.rasmus.gui.YesNoDialog;
import visl.rasmus.hitserver.client.HitXtraClient;
import visl.rasmus.hitserver.data.Sentence;
import visl.rasmus.hitserver.data.TreeNode;
import visl.rasmus.hitserver.server.HitServerInterface;

public class DiagnoseServer extends Frame {
    protected HitXtraClient hc;
    protected Frame parent;
    protected TextArea display;
    protected TextField command;

    public DiagnoseServer(int port, Frame parent, boolean err) {
	hc = new HitXtraClient("localhost", port, null, null);
	this.parent = parent;

	Panel p = null, p2 = null;
	Button b = null;
	Separator sep = null;
	Label l = null;

	//ScrollablePanel sp = new ScrollablePanel(root);
	p = new Panel();
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.weightx = 1;
	c.weighty = 1;
	c.gridwidth = 1;
	p.setLayout(gbl);
	
	l = new Label("Diagnose:");
	gbl.setConstraints(l, c);
	p.add(l);
	b = new Button("Get Version");
	gbl.setConstraints(b, c);
	p.add(b);
	b = new Button("See Error Logs");
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(b, c);
	p.add(b);
	b = new Button("Run Diagnose");
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(b, c);
	p.add(b);

	sep = new Separator();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(sep, c);
	p.add(sep);

	c.gridwidth = 1;
	l = new Label("Manage:");
	gbl.setConstraints(l, c);
	p.add(l);
	b = new Button("Restart Server");
	gbl.setConstraints(b, c);
	p.add(b);
	b = new Button("Shutdown Server");
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(b, c);
	p.add(b);
	b = new Button("Kill Server");
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(b, c);
	p.add(b);

	sep = new Separator();
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(sep, c);
	p.add(sep);

	//b = new Button("Start Server"));;gbl.setConstraints(b, c);p.add(b);
	c.gridwidth = 1;
	l = new Label("Debug:");
	gbl.setConstraints(l, c);
	p.add(l);
	b = new Button("ExecCommand");
	gbl.setConstraints(b, c);
	p.add(b);
	command = new TextField(20);
	c.gridwidth = GridBagConstraints.RELATIVE;
	gbl.setConstraints(command, c);
	p.add(command);
	b = new Button("Debug Databases");
	c.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(b, c);
	p.add(b);
	
	add("North", p);

	//p = new Panel();
	display = new TextArea(20, 70);
	display.setFont(new Font("Courier", Font.PLAIN, 14));
	//p.add("Center", display);
	add("Center", display);

	p = new Panel();
	b = new Button("Quit");
	p.add(b);
	add("South", p);
    }
    public boolean handleEvent(Event e) {
	if (e.id == Event.WINDOW_DESTROY) {
	    this.dispose();
	    parent.show();
	    return true;
	}
	if (e.target instanceof Button &&
	    e.id == Event.ACTION_EVENT) {
	    Button b = (Button)e.target;
	    if (b.getLabel().equals("Quit")) {
		postEvent(new Event(this, Event.WINDOW_DESTROY, null));
		return true;
	    } else if (b.getLabel().equals("Get Version")) {
		Enumeration en = hc.sendQuery(HitServerInterface.GET_VERSION).elements();
		String version = null;
		if (en.hasMoreElements()) version = (String)en.nextElement();
		display.setText("Version is: [" + version + "]\n");
	    } else if (b.getLabel().equals("See Error Logs")) {
		FileDialog fd = new FileDialog(this, "Browse log-files.", 
					       FileDialog.LOAD);
		fd.setDirectory("./logs");
		fd.setFilenameFilter(new FilenameFilter() {
			public boolean accept(File dir, String name) {
			    return name.endsWith(".log");
			}
		    });
		fd.show();
		String s = fd.getFile();
		if (s != null) printFile(fd.getDirectory()+s);
		else display.setText("No log-file selected!");
	    } else if (b.getLabel().equals("Run Diagnose")) {
		doDiagnose();
	    } else if (b.getLabel().equals("ExecCommand")) {
		display.setText("");
		printVector(hc.sendQuery(command.getText()));
		//printVector(doCommand(command.getText()));
	    } else if (b.getLabel().equals("Restart Server")) {
		doRestartServer();
	    } else if (b.getLabel().equals("Shutdown Server")) {
		doShutdownServer();
	    } else if (b.getLabel().equals("Kill Server")) {
		YesNoDialog ynd = new YesNoDialog(this, "Really kill HitServer?");
		TextArea ta = new TextArea();
		ta.setText("It is strongly recommended to try the less violent approaches,\nbefore killing the server this way!\nIf you just want to stop the server from running,\nuse the 'Shutdown Server' option if possible!\n\nAre you sure you want to kill the HitServer?");
		ta.setEditable(false);
		ynd.add("Center", ta);
		ynd.pack();
		ynd.show();
		boolean res = ynd.isOk();
		ynd.dispose();
		if (res) {
		    doKillServer();
		}
	    } else if (b.getLabel().equals("Debug Databases")) {
		DebugDatabases dd = new DebugDatabases(this, hc);
		dd.show();
		this.hide();
	    }
	}
	return false;
    }
    /** Executes a command and returns a vector containing the lines that
     * the command printed to stdout */
    protected Vector doCommand(String com) {
	Vector v = new Vector();
	Process p = null;
	try {
	    p = Runtime.getRuntime().exec(com);
	    String line;
	    DataInputStream dis = 
		new DataInputStream(p.getInputStream());
	    while ((line = dis.readLine()) != null)
		v.addElement(line);
	    p.waitFor();
	} catch (Exception x) {
	    v.addElement("Exception: " + x);
	    //x.printStackTrace();
	} finally {
	    if (p != null) p.destroy();
	}
	return v;
    }
    protected void printVector(Vector v) {
	StringBuffer buff = new StringBuffer();
	for (Enumeration en = v.elements(); en.hasMoreElements();) {
	    buff.append(en.nextElement()+"\n");
	}
	display.append(buff.toString());
    }
    protected void printFile(String filename) {
	DataInputStream dis = null;
	String s = null;
	StringBuffer buff = new StringBuffer();
	try {
	    dis = new DataInputStream(new FileInputStream(filename));
	    display.setText("");
	    while ((s = dis.readLine()) != null)
		buff.append(s + "\n");
	    dis.close();
	} catch (Exception x) {
	    display.setText("Exception: " + x + "\n");
	} finally {
	    try { 
		if (dis != null) dis.close();
	    } catch (IOException _) {}
	}
	display.setText(buff.toString());
    }
    protected void doDiagnose() {
	int err = 0;
	String s = null;
	display.setText("");
	display.append("Testing connection to the HitServer...");
	try {
	    hc.connect();
	    hc.close();
	} catch (Exception e) {
	    err = 1;
	    display.append("error!\n - " + e + "\n");
	}
	if (err==0) display.append("done!\n");
	
	// Check version information (also communication with server).
	if (err==0) {
	    display.append("Retreiving version: ");
	    Enumeration en = hc.sendQuery(HitServerInterface.GET_VERSION).elements();
	    if (en.hasMoreElements()) s = (String)en.nextElement();
	    if (s == null || !s.startsWith("HitServer")) err += 2;
	    display.appendText(s+"\n");
	}
	
	// Retreive the languages
	String sa[] = null;
	if ((err & 0x01) == 0) {
	    display.append("Retreiving languages: ");
	    sa = hc.getLanguages();
	    if (sa == null) err += 4;
	    for (int i = 0; err == 0 && i < sa.length; i++)
		display.append(sa[i]+ (i != sa.length-1 ? ", " : "\n"));
	}

	// Retreive a random sentence from each language
	if ((err & 0x01) == 0) {
	    display.append("Retreiving random sentences:\n");
	    for (int i = 0; i < sa.length; i++) {
		s = null;
		hc.setLanguage(sa[i]);
		Sentence sc = hc.getRandomSentence();
		if (sc != null) s = sc.toString();
		if (s == null || !s.startsWith("Sentence[")) err = err | 0x08;
		display.append("  "+ sa[i] + ": " +s+ "\n");
	    }
	}

	// Retreiving random trees.
	if ((err & 0x01) == 0) {
	    display.append("Retreiving random trees:\n");
	    for (int i = 0; i < sa.length; i++) {
		s = null;
		hc.setLanguage(sa[i]);
		Vector v = hc.getRandomTrees(0,255,1);
		TreeNode n = null;
		if (v != null && v.size() > 0) n = (TreeNode)v.firstElement();
		if (n != null) s = n.toString();
		if (n == null || !s.startsWith("Tree[")) err = err | 0x10; 
		display.append("  "+sa[i]+": "+s+"\n");
	    }
	}
	
	if (err != 0) {
	    display.append("\nThere were errors:\n");
	    if ((err & 0x01) != 0) 
		display.append(" - No connection to the server!\n");
	    if ((err & 0x02) != 0) 
		display.append(" - Wrong version!\n"+
			       "   (perhaps port is occupied by another protocol)\n");
	    if ((err & 0x04) != 0) 
		display.append(" - Error retreiving language-list!\n");
	    if ((err & 0x08) != 0) 
		display.append(" - Not all languages returned a random sentence!\n");
	    if ((err & 0x10) != 0) 
		display.append(" - Not all languages returned a random tree!\n");
	} else {
	    display.append("\nNo errors encountered. Server is running correctly!\n");
	}
	
    }
    /** This method makes the HitServer restart (on its own).*/
    protected void doRestartServer() {
	display.setText("Restarting the server...");
	hc.sendCommand("RESTART");
	try { Thread.sleep(500); } catch (Exception _) {}
	try {
	    hc.connect();
	    hc.close();
	    display.append("done!\n");
	} catch (Exception _) {
	    display.append("done!\n");
	}
	display.append("Wait a couple of seconds, and try the features that\n"+
		       "should have been corrected after the rebooting of the"+
		       "server.\n");
    }
    /** This method tries to shutdown the server in a mannerly fashion
     * (asking the server to shutdown on its own).*/
    protected void doShutdownServer() {
	display.setText("Shutting down server...");
	hc.sendCommand(HitServerInterface.QUIT);
	display.append("done!\n");
	display.append("NOTE: The server should now have been shut down.\n"+
		       "Run 'Diagnose Server' to make sure.\n"+
		       "IMPORTANT: The server should be restarted manually\n"+
		       "in the near future, since many applications depend\n"+
		       "on it!");
    }
    /** This function kills all the processes that seem to belong to the
     * HitServer. */
    protected void doKillServer() {
	display.setText("");
	Vector v = null;
	int pid = -1, opid = -1;
	while (v == null || v.size() > 0) {
	    v = doCommand("./scripts/getAllGhosts2");
	    if (v.size() > 0) {
		opid = pid;
		pid = -1;
		String s = (String)v.firstElement();
		if (s.startsWith("Exception:")) {
		    display.append(s+"\n");
		} else {
		    try {
			pid = Integer.parseInt(s.substring(0,s.indexOf(' ')));
		    } catch (NumberFormatException e) {
			display.append("Not a number: [" + s + "]\n");
		    }
		}
		if (pid != -1 && opid != pid) {
		    // Kill pid.
		    display.append("Killing pid [" + pid + "]:" + s + "\n");
		    doCommand("kill " + pid);
		} else {
		    display.append("The pid ["+pid+"] wont be killed!\n");
		    v = new Vector();
		}
	    } else {
		display.append("No ghost-processes was found!\n");
	    }
	}
	// Wait for a sec.
	try { Thread.sleep(500); } catch (Exception _) {}
	if ((v = doCommand("./getAllGhosts2")).size() == 0) {
	    display.append("The HitServer has successfully been slayed!\n");
	} else {
	    display.append("The HitServer survives - something went wrong!\n");
	    display.append("Output: [" + v + "]\n");
	}
	
    }
}


