package visl.rasmus.resources;

import java.io.DataInputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
* ResourceBundle-like class containing a dictionary of titles, menus, messages, etc..
* See the localization HOWTO on how to define the translations.
*@author Martin Carlsen
*/
public class Dict {
	/**
	 * Generic dictionary.
	 * Contains ("English string", "Localized string") pairs of strings.
	 */
	static Hashtable dictionary;
	static Hashtable languages;
	static String url;
	protected static boolean debug = false;
	/**	
	 * Initializes tables.
	 */
	public static void localize() {
		String[][] defaultdictionary = {
		};
		String[][] langs = null;
		if (url != null) {
			try {
				langs =
					readSet(
						url.substring(0, url.lastIndexOf('/') + 1)
							+ "langs.lst");
			} catch (Exception x) {
				System.out.println(
					"Could not retreive: "
						+ url.substring(0, url.lastIndexOf('/') + 1)
						+ "langs.lst");
			}
		}
		initTables(defaultdictionary, langs);
	}

	/**
	 * Read translations from url.
	 *@param dictUrl url to read translations from.
	 *@exception MalformedURLException if any of the URL
	 */
	public static void localize(String dictUrl)
		throws MalformedURLException, IOException {
		url = dictUrl;
		String[][] dict = readSet(dictUrl);
		String[][] langs = null;
		try {
			langs =
				readSet(
					url.substring(0, url.lastIndexOf('/') + 1) + "langs.lst");
		} catch (Exception x) {
			System.out.println(
				"Could not retreive: "
					+ url.substring(0, url.lastIndexOf('/') + 1)
					+ "langs.lst");
		}
		initTables(dict, langs);
	}

	/**
	 * Read a name=value set from an url
	 * Format of the file pointed to: ({English text}'='{localized text}'\n')*
	 */
	static String[][] readSet(String url)
		throws MalformedURLException, IOException {
		return readSet(new URL(url));
	}

	/**
	 * Read a name=value set from an url
	 * Format of the file pointed to: ({English text}'='{Localized text}'\n')*
	 */
	static String[][] readSet(URL url)
		throws MalformedURLException, IOException {
		Vector set = new Vector();
		String s = null;
		//BufferedReader d = null;
		//d = new BufferedReader(new InputStreamReader(url.openStream())); // use a BufferedReader for input.
		DataInputStream d = null;
		d = new DataInputStream(url.openStream());
		s = d.readLine(); // read the first line
		String abbr, full; // abbreviated and full name of symbol
		int equ; // position of the '=' character
		int linenum = 1;
		while (s != null) { // while not EOF
			equ = s.indexOf('=');
			if (s.startsWith("//")) {
				if (debug)
					System.out.println(linenum + ":comment!");
			} else if (equ != -1) {
				abbr = s.substring(0, equ);
				full = s.substring(equ + 1, s.length());
				// Add the (abbr,full name) pair to the set of symbols
				if (full.length() > 0)
					set.addElement(new String[] { abbr, full });
			} else {
				if (debug)
					System.out.println(linenum + ":no input!");
			}
			s = d.readLine(); // read the next line.
			linenum++;
		}

		int n = set.size(); // number of symbols
		String[][] symbols = new String[n][];
		for (int i = 0; i < n; i++) {
			symbols[i] = (String[]) set.elementAt(i);
		}
		return symbols;
	}

	/**
	 * Get a translated String.
	 *@param key English string to get local version of.
	 *@return Local string. Or English string if there is no local version.
	 */
	public static String get(String key) {
		Object o = dictionary.get(key);
		if (o == null)
			return key;
		return (String) o;
	}

	/**
	 * Initialize tables.
	 * Called by the localize method.
	 *@param dict Array of {"English string", "localized string"} dictionary pairs.
	 */
	static void initTables(String[][] dict, String[][] langs) {
		dictionary = new Hashtable();
		languages = new Hashtable();
		for (int i = 0; i < dict.length; i++)
			dictionary.put(dict[i][0], dict[i][1]);
		if (langs != null)
			for (int i = 0; i < langs.length; i++)
				languages.put(langs[i][0], langs[i][1]);
	}
	static public void setLanguage(String abbr) {
		try {
			localize(url.substring(0, url.lastIndexOf('.') + 1) + abbr);
			System.out.println("Changed dictionary to : " + url);
		} catch (IOException x) {
			System.out.println("Failed to change dictionary to : " + url);
		}
	}
	static public Enumeration getLanguages() {
		return languages.keys();
	}
	static public String getLanguageName(String abbr) {
		return (String) languages.get(abbr);
	}
	static public boolean isInitialized() {
		return dictionary != null;
	}
}
