package visl.rasmus.resources;

import java.applet.Applet;
import java.awt.Canvas;
import java.awt.Image;
import java.awt.image.ImageObserver;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Hashtable;

/**
* ResourceBundle-like class for encapsulation of the images used by a given 
* language. The images are loaded into a hashtable when needed.
* @author Rasmus Lock Larsen, VISL, Nov. 2000
*/
public class Images extends Canvas implements ImageObserver {
    /** Debug flag */
    static protected boolean debug = false;
    private Images() {
    }
    public boolean imageUpdate(Image img, int flag, 
			       int x, int y, int w, int h) {
	Images.flag = flag;
	if ((flag & ERROR) != 0) {
	    errorflag = true;
	} else if ((flag & WIDTH) != 0 &&
		   (flag & HEIGHT) != 0) {
	    ready = true;
	}
	return true;
    }
    static final Images imgs = new Images();
    static final int RETRY = 10;
    static boolean errorflag, ready;
    static int flag;

    static Applet applet = null;
    static URL imageURL = null;
    
    /*
     * The {type, comment} pairs as Hashtable.
     */
    static Hashtable images = new Hashtable();
    static Hashtable nullspace = new Hashtable();
	
    /** Returns the size of the image (a square)
     * NB: The name is in need of a better one (size() and getSize() is taken)
     */
    public static void setApplet(Applet aplt, String imagebase) {
	flush();
	applet = aplt;
	try {
	    imageURL = new URL(imagebase);
	} catch (MalformedURLException e) {
	    e.printStackTrace();
	    applet = null;
	}
    }
    /** Flushes the Image database... */
    public static void flush() {
	if (debug) System.out.println("Flushing images...");
	images.clear();
	nullspace.clear();
    }
    /** Returns the image that corresponds to the abbr. IF the Images has been
     * enabled (see Miscellaneous, "symbols") */
    public static Image getImage(String filename) {
	if (images.containsKey(filename)) return (Image)images.get(filename);
	if (nullspace.containsKey(filename)) {
	    int s = ((Integer)nullspace.get(filename)).intValue();
	    if (s >= RETRY) 
		nullspace.remove(filename);
	    else
		nullspace.put(filename, new Integer(s+1));
	    return null;
	}
	if (applet != null) {
	    Image image = null;
	    if (debug) System.out.print("Loading image: " + filename + "\t");
	    image = applet.getImage(imageURL, filename);
	    errorflag = false;
	    ready = false;
	    while (!errorflag && !ready && !imgs.prepareImage(image, imgs)) {
		try { Thread.sleep(5); } catch (InterruptedException e) {}
		if (debug) System.out.print(".");
	    }
	    if (errorflag) {
		if (debug) System.out.println("error!");
		nullspace.put(filename, new Integer(0));
	    } else {
		if (debug) System.out.println("done!");
		images.put(filename, image);
		return image;
	    }
	}
	return null;
    }
    public static void setDebugging(boolean b) { debug = b; }
}

