package visl.rasmus.utils;

import java.util.Enumeration;
import java.util.Vector;

public class Timer {
    Vector listeners;
    long timein, timeout, elapsed;
    long interval, time_out;
    TimerThread tt;
    boolean paused;
    /** A timer that calls all the TimerListeners every <CODE>interval</CODE>
     * milliseconds, can also be used as a time out checker. If the reset() method is called,
     * the listening TimerListeners are only being tick()'ed iff the time_out is true (no
     * reset() calls between the last ticks).
     * @param interval time between timer calls in milliseconds. */
    public Timer(long interval) {
	paused = false;
	time_out = 0;
	elapsed = 0;
	this.interval = interval;
	listeners = new Vector();
	tt = new TimerThread();
	tt.setDaemon(true);
    }
    public void addListener(TimerListener tl) {
	listeners.addElement(tl);
    }
    public void removeListener(TimerListener tl) {
	listeners.removeElement(tl);
    }
    public void pause() { paused = true; }
    public void stop() { 
	//tt.stop(); 
	tt.interrupt();
    }
    public void start() { 
	if (tt.isAlive() && paused) paused = false;
	else tt.start(); 
    }
    public boolean running() { return tt.isAlive(); }
    public void setInterval(long intv) {
	interval = intv;
    }
    public long getInterval() { return interval; }
    public void setTimeOut(long timeo) { time_out = timeo; }
    public long getTimeOut() { return time_out; }
    public void reset() { 
	elapsed = 0;
	if (paused) paused = false;
    }
    public class TimerThread extends Thread {
	public void run() {
	    try {
		timeout = System.currentTimeMillis();
		while (true) {
		    timein = System.currentTimeMillis();
		    if (time_out > 0) elapsed += timein - timeout;
		    if (!paused && (elapsed >= time_out)) {
			for (Enumeration en = listeners.elements(); 
			     en.hasMoreElements(); ) {
			    ((TimerListener)en.nextElement()).tick();
			}
		    }
		    timeout = System.currentTimeMillis();
		    if (timeout - timein < interval)
			sleep(interval - (timeout-timein));
		}
	    } catch (InterruptedException x) {
		System.out.println("Timer abnormally interrupted!");
	    }
	}
    }
}
